
#include "datatypes.h"
#include "can.h"
#include "can_user.h"



// timings for CAN1 and CAN2
static const CANTiming_t  Timing_CANx[2] = {
_80M_500K_80____4M_80_ISO,	  // CAN1
_80M_500K_80____4M_80_ISO	  // CAN2
};





// CAN_UserRead
// This wrapper is used to pre-process some incoming data.
CANResult_t  CAN_UserRead ( CANHandle_t  hBus, void  *buff)
{
	CANResult_t  ret;
	CANBuffer_t  *rx_buff;
	
	
	ret = CAN_ERR_RX_EMPTY;
	rx_buff = buff;
	
	if ( CAN_Read ( hBus, rx_buff) == CAN_ERR_OK)
	{
		// buffer read from CANx. Check type of buffer.
		switch ( rx_buff->bufftype)
		{
			case CAN_BUFFER_STATUS:
				// the buffer is a status notification from the CAN controller.
				if ( rx_buff->status.bus_status)
				{
					// CAN controller not involved into bus activities
					
					// uninitialize (this includes a TX-path flush)
					CAN_UnInitialize ( hBus);
					
					// initialize CAN controller
					CAN_Initialize ( hBus, &Timing_CANx[hBus]);
				}
				break;
				
				
			case CAN_BUFFER_RX_MSG:
				// the buffer is a receive message. Forward to application.
				ret = CAN_ERR_OK;
				break;
				
				
			case CAN_BUFFER_CRITICAL:
				// receive queue level was critical. Data might be lost.
				break;
		}
	}
	
	return ret;
}



// init CAN1 and CAN2
void  CAN_UserInit ( void)
{
	// init CAN1
	CAN_Initialize ( CAN_BUS1, &Timing_CANx[CAN_BUS1]);
	
	// receive all 11 bit standard CAN-IDs
	CAN_FilterAdd ( CAN_BUS1, CAN_MSGTYPE_STANDARD, 0x000, 0x7FF);
	
	
	
	// init CAN2
	CAN_Initialize ( CAN_BUS2, &Timing_CANx[CAN_BUS2]);
	
	// receive all 11 bit standard CAN-IDs
	CAN_FilterAdd ( CAN_BUS2, CAN_MSGTYPE_STANDARD, 0x000, 0x7FF);
}

