
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"
#include "eeprom.h"


// abstract:
// This example shows the usage of the MCU's internal EEPROM memory. HW_Init() will initialize
// the MCU so you can use EEPROM_.. functions immediately. The MCU will provide 4032 bytes of
// EEPROM memory. If you need more storage, take a look to the SPI Flash example for using the
// additional on-board 32 MBit SPI Flash.


// identifier is needed by PCANFlash.exe -> do not delete
const char Ident[] __attribute__ ((used)) = { "PCAN-Router_FD"};


// variables for LED toggle
static uint8_t LED_toggleCAN1;
static uint8_t LED_toggleCAN2;





// main_greeting()
// transmit a message at module start
static void  main_greeting ( void)
{
	CANTxMsg_t  Msg;
	
	
	Msg.bufftype = CAN_BUFFER_TX_MSG;
	Msg.dlc      = CAN_LEN8_DLC;
	Msg.msgtype  = CAN_MSGTYPE_STANDARD;
	Msg.id       = 0x123;
	
	// we will read 8 bytes from eeprom address 100h
	EEPROM_Read ( 0x100, &Msg.data8[0], 8);
	
	// Send Msg
	CAN_Write ( CAN_BUS1, &Msg);
}





// main()
// entry point from cr_startup_lpc40xx.c
int  main ( void)
{
	// init hardware and timer 0. Timer 0 is free running
	// with 1 us resolution without any IRQ.
	HW_Init();
	
	// init CAN
	CAN_UserInit();
	
	// set green LEDs for CAN1 and CAN2
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	
	// send the greeting message
	main_greeting();
	
	// main loop
	while ( 1)
	{
		CANRxMsg_t  RxMsg;
		
		
		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			
			
			// catch ID 48Ah from CAN1 to save new data to the eeprom
			if ( RxMsg.id == 0x48A  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				// write new data to eeprom. Next power cycle will show a modified
				// greeting message.
				EEPROM_Write ( 0x100, &RxMsg.data8[0], 8);
				EEPROM_FlushCache();
			}
			
			else
			{
				// forward message to CAN2
				CAN_Write ( CAN_BUS2, &RxMsg);
			}
		}
		
		
		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			// forward message to CAN1
			CAN_Write ( CAN_BUS1, &RxMsg);
		}
	}
}
