
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"


// abstract:
// Here we will modify some signals from different CAN messages. These are operations
// like signal limiting or value exchange.


// identifier is needed by PCANFlash.exe -> do not delete
const char Ident[] __attribute__ ((used)) = { "PCAN-Router_FD"};


// variables for LED toggle
static uint8_t LED_toggleCAN1;
static uint8_t LED_toggleCAN2;

// variable for signal
static uint8_t  Temp190 = 0;





// main_greeting()
// transmit a message at module start
static void  main_greeting ( void)
{
	CANTxMsg_t  Msg;
	
	
	Msg.bufftype = CAN_BUFFER_TX_MSG;
	Msg.dlc      = CAN_LEN8_DLC;
	Msg.msgtype  = CAN_MSGTYPE_STANDARD;
	Msg.id       = 0x123;
	
	Msg.data32[0] = 0x67452301;
	Msg.data32[1] = 0xEFCDAB89;
	
	// Send Msg
	CAN_Write ( CAN_BUS1, &Msg);
}





// main()
// entry point from cr_startup_lpc40xx.c
int  main ( void)
{
	// init hardware and timer 0. Timer 0 is free running
	// with 1 us resolution without any IRQ.
	HW_Init();
	
	// init CAN
	CAN_UserInit();
	
	// set green LEDs for CAN1 and CAN2
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	
	// send the greeting message
	main_greeting();
	
	// main loop
	while ( 1)
	{
		CANRxMsg_t  RxMsg;
		
		
		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			
			
			// catch ID 120h from CAN1 to modify the ID and message type
			if ( RxMsg.id == 0x120  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				RxMsg.id = 0x4ED8;
				RxMsg.msgtype = CAN_MSGTYPE_EXTENDED;
			}
			
			// catch ID 150h and exchange two data bytes
			else if ( RxMsg.id == 0x150  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				uint8_t tmp;
				
				
				tmp = RxMsg.data8[0];
				RxMsg.data8[0] = RxMsg.data8[1];
				RxMsg.data8[1] = tmp;
			}
			
			// catch ID 180h and limit a signal to a maximum
			else if ( RxMsg.id == 0x180  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				uint16_t  speed;
				
				
				// get the signal (intel format)
				speed = ( RxMsg.data32[0] >> 12) & 0x1FFF;
				
				// limit value
				if ( speed > 6200)
				 { speed = 6200;}
				
				// replace the original value
				RxMsg.data32[0] &= ~( 0x1FFF << 12);
				RxMsg.data32[0] |= speed << 12;
			}
			
			// catch ID 190h and save a signal
			else if ( RxMsg.id == 0x190  &&  RxMsg.msgtype == CAN_MSGTYPE_STANDARD)
			{
				Temp190 = ( RxMsg.data32[0] >> 3) & 0x3F;
				goto CheckCAN2;	// skip the routing
			}
			
			// catch ID 44E0h and send the saved signal from ID 190h
			else if ( RxMsg.id == 0x44E0  &&  RxMsg.msgtype == CAN_MSGTYPE_EXTENDED)
			{
				RxMsg.data32[0] &= ~( 0x3F << 3);
				RxMsg.data32[0] |= Temp190 << 3;
			}
			
			// forward message to CAN2
			CAN_Write ( CAN_BUS2, &RxMsg);
		}
		
		
		CheckCAN2:
		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			// forward message to CAN1
			CAN_Write ( CAN_BUS1, &RxMsg);
		}
	}
}
