

MEMORY
{
  /* Define each memory region */
  RAM_64k (rwx) : ORIGIN = 0x10000000, LENGTH = 0x10000 		/* 64k */
  RAM_32k (rwx) : ORIGIN = 0x20000000, LENGTH = 0x08000 		/* 32k */
  RO_FLASH (rx) : ORIGIN = 0x00005000, LENGTH = 0x7B000		/* Flash: 512k ( first 20K for PCAN-Flash loader */
}
  /* Define a symbol for the top of each memory region */
  __TOP_RAM_64k  = 0x10000000 + 0x10000;
  __TOP_RAM_32k  = 0x20000000 + 0x08000;
  __TOP_RO_FLASH = 0x00005000 + 0x7B000;


/* Entrypoint */
ENTRY(ResetISR)

SECTIONS
{

	__code_checksum = 0 - (_vStackTop + ResetISR + 1 + NMI_Handler + 1 + HardFault_Handler + 1 + MemManage_Handler + 1 + BusFault_Handler + 1 + UsageFault_Handler + 1);
	
	/* first setcion is the main text section for code  */	
	/*.text ALIGN(8192) : ALIGN(4)*/
	.text : ALIGN(4)
	{
		FILL(0xff)
		KEEP(*(.isr_vector))
		
		/* Global Section Table */
		. = ALIGN(4) ;
		__section_table_start = .;
		__data_section_table = .;
		LONG(LOADADDR(.data));
		LONG(    ADDR(.data)) ;
		LONG(  SIZEOF(.data));
		LONG(LOADADDR(.data_RAM2));
		LONG(    ADDR(.data_RAM2)) ;
		LONG(  SIZEOF(.data_RAM2));
		__data_section_table_end = .;
		__bss_section_table = .;
		LONG(    ADDR(.bss));
		LONG(  SIZEOF(.bss));
		LONG(    ADDR(.bss_RAM2));
		LONG(  SIZEOF(.bss_RAM2));
		__bss_section_table_end = .;
		__section_table_end = . ;
		/* End of Global Section Table */
		

		*(.after_vectors*)
		
		*(.text*)
		*(.rodata .rodata.*)
		. = ALIGN(4);
		
	} > RO_FLASH
		
	_etext = .;


	/*
	 * for exception handling/unwind - some Newlib functions (in common
	 * with C++ and STDC++) use this. 
	 */
	.ARM.extab : ALIGN(4)
	{
		*(.ARM.extab* .gnu.linkonce.armextab.*)
	} > RAM_64k 
	__exidx_start = .;
	
	.ARM.exidx : ALIGN(4)
	{
		*(.ARM.exidx* .gnu.linkonce.armexidx.*)
	} > RAM_64k 
	__exidx_end = .;
	
	/* MAIN DATA SECTION */
	/*

	.uninit_RESERVED : ALIGN(4)
	{
		KEEP(*(.bss.$RESERVED*))
		. = ALIGN(4) ;
		_end_uninit_RESERVED = .;
	} > RAM_64k 
	*/
	
	.data : AT(_etext)  ALIGN(4) 
	{
		FILL(0xff)
		_data = .;
		*(vtable)
		*(.data*)
		. = ALIGN(4) ;
		_edata = .;
	} > RAM_64k

	.data_RAM2 : AT(_edata) ALIGN(4)
	{
	   FILL(0xff)
		*(.data.$RAM2*)
		*(.data.$RAM_32k*)
	   . = ALIGN(4) ;
	} > RAM_32k 
	
	/* MAIN BSS SECTION */
	.bss : ALIGN(4)
	{
		_bss = .;
		*(.bss*)
		*(COMMON)
		. = ALIGN(4) ;
		_ebss = .;
		PROVIDE(end = .);
	} > RAM_64k
			
	.bss_RAM2 : ALIGN(4)
	{
		*(.bss.$RAM2*)
		*(.bss.$RAM_32k*)
	   . = ALIGN(4) ;
	} > RAM_32k
		
	/*	
	.noinit_RAM2 (NOLOAD) : ALIGN(4)
	{
		*(.noinit.$RAM2*)
		*(.noinit.$RAM_32k*)
	   . = ALIGN(4) ;
	} > RAM_32k
	*/
	/* DEFAULT NOINIT SECTION */
	/*
	.noinit (NOLOAD): ALIGN(4)
	{
		_noinit = .;
		*(.noinit*) 
	 	. = ALIGN(4) ;
		_end_noinit = .;
	} > RAM_64k 
	*/

	PROVIDE(_pvHeapStart = .);
	PROVIDE(_vStackTop = __TOP_RAM_64k - 0);
}

