
#include "datatypes.h"
#include "hardware_user.h"



// data for SPIFI Flash interface
SPIFI_FAM_NODE_T  *pFamNode;
SPIFI_HANDLE_T  *pSpifi;
static uint32_t  SpifiMem[16];




// 
void  HW_UserInit ( void)
{
	// init SPIFI
	spifiInit ( 0x20094000, 1);

	pFamNode = spifiRegisterFamily ( spifi_REG_FAMILY_CommonCommandSet);

	// Former Spansion Flash (32 MBit)
	// Quad-IO *not* enabled during PCAN-Router FD production.
	{
		static const SPIFI_DEVICE_DATA_T pData = {
			"S25FL132K",
			{{0x01, 0x40, 0x16}, 0, {0}},	/* JEDEC ID, extCount, ext data  */
			(SPIFI_CAP_DUAL_READ | SPIFI_CAP_QUAD_READ | SPIFI_CAP_FULLLOCK | SPIFI_CAP_NOBLOCK | SPIFI_CAP_SUBBLKERASE),	/* does NOT support Quad Write */
			64,					/* # of blocks */
			0x10000,				/* block size */
			1024,					/* # of sub-blocks */
			0x1000,					/* sub-block size */
			256,					/* page size */
			16128,					/* max single read bytes */
			50,				/* max clock rate in MHz */
			97,				/* max read clock rate in MHz */
			97,				/* max high speed read clock rate in MHz */
			97,				/* max program clock rate in MHz */
			97,				/* max high speed program clock rate in MHz */
			FX_spifiDeviceDataInitDeinitS25FL164K,	/* (Fx Id) device init / deInit */
			FX_spifiDeviceDataClearStatusNone,	/* (Fx Id) No persistent status */
			FX_spifiDeviceDataGetStatusS25FL164K,	/* (Fx Id) getStatus */
			FX_spifiDeviceDataSetStatusS25FL164K,	/* (Fx Id) setStatus */
			FX_spifiDeviceDataSetOptsQuadModeBit9,	/* (Fx Id) to set/clr options */
			FX_spifiDeviceInitReadCommand,	/* (Fx Id) to get memoryMode Cmd */
			FX_spifiDeviceInitWriteCommand	/* (Fx Id) to get program Cmd */
		};
		static SPIFI_DEV_NODE_T data;			/* Create persistent node */

		data.pDevData = &pData;					/* save the data in the node */
		spifiDevRegister ( pFamNode, &data);	/* Register the new device */
	}
	
	
	// New ISSI Flash (64 MBit)
	// Quad-IO enabled during PCAN-Router FD production.
	{
		static const SPIFI_DEVICE_DATA_T pData = {
			"IS25LP064A",
			{{0x9D, 0x60, 0x17}, 0, {0}},	/* JEDEC ID, extCount, ext data  */
			(SPIFI_CAP_DUAL_READ | SPIFI_CAP_QUAD_READ | SPIFI_CAP_QUAD_WRITE | SPIFI_CAP_FULLLOCK | SPIFI_CAP_NOBLOCK | SPIFI_CAP_SUBBLKERASE),
			128,					/* # of blocks */
			0x10000,				/* block size */
			2048,					/* # of sub-blocks */
			0x1000,					/* sub-block size */
			256,					/* page size */
			16128,					/* max single read bytes */
			50,				/* max clock rate in MHz */
			50,				/* max read clock rate in MHz */
			97,				/* max high speed read clock rate in MHz */
			97,				/* max program clock rate in MHz */
			97,				/* max high speed program clock rate in MHz */
			FX_spifiDeviceDataInitDeinit_ISSI,	/* (Fx Id) device init / deInit */
			FX_spifiDeviceDataClearStatusNone,	/* (Fx Id) No persistent status */
			FX_spifiDeviceDataGetStatusIS25LP064A,	/* (Fx Id) getStatus */
			FX_spifiDeviceDataSetStatusIS25LP064A,	/* (Fx Id) setStatus */
			FX_spifiDeviceDataSetOptsQuadModeReady,	/* (Fx Id) to set/clr options */
			FX_spifiDeviceInitReadCommand,	/* (Fx Id) to get memoryMode Cmd */
			FX_spifiDeviceInitWriteCommand	/* (Fx Id) to get program Cmd */
		};
		
		static SPIFI_DEV_NODE_T data;			/* Create persistent node */

		data.pDevData = &pData;					/* save the data in the node */
		spifiDevRegister ( pFamNode, &data);	/* Register the new device */
	}

	pSpifi = spifiInitDevice ( SpifiMem, sizeof ( SpifiMem), 0x20094000, 0x28000000);

	if ( pSpifi->pInfoData->pId->mfgId[0] == 0x9D)
	{
		// quad read/write only for ISSI. Quad-IO enabled during production.
		// If you want to be backward compatible to former flash, uncomment next
		// line.
		spifiDevSetOpts ( pSpifi, SPIFI_OPT_USE_QUAD, 1);
	}
}

