
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"


// abstract:
// This example shows the routing of CAN messages from CAN1 to CAN2.
// The messages are not modified.
// Reception of CAN-ID 45Dh on CAN1 will put the module into power
// down mode e.g. for remote wake-up. Send a message to the network
// from another station to wake-up the router. The voltage regulator
// is fully turned off. See PCAN-Router FD user manual for wake-up
// capability (depends on HW version / serial number).


// identifier is needed by PCANFlash.exe -> do not delete
const char Ident[] __attribute__ ((used)) = { "PCAN-Router_FD"};


// variables for LED toggle
static uint8_t LED_toggleCAN1;
static uint8_t LED_toggleCAN2;





// main_greeting()
// transmit a message at module start
static void  main_greeting ( void)
{
	CANTxMsg_t  Msg;
	
	
	Msg.bufftype = CAN_BUFFER_TX_MSG;
	Msg.dlc      = CAN_LEN8_DLC;
	Msg.msgtype  = CAN_MSGTYPE_STANDARD;
	Msg.id       = 0x123;
	
	Msg.data32[0] = 0x67452301;
	Msg.data32[1] = 0xEFCDAB89;
	
	// Send Msg
	CAN_Write ( CAN_BUS1, &Msg);
}





// main()
// entry point from cr_startup_lpc40xx.c
int  main ( void)
{
	// init hardware and timer 0. Timer 0 is free running
	// with 1 us resolution without any IRQ.
	HW_Init();
	
	// init CAN
	CAN_UserInit();
	
	// set green LEDs for CAN1 and CAN2
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	
	// send the greeting message
	main_greeting();
	
	// main loop
	while ( 1)
	{
		CANRxMsg_t  RxMsg;
		
		
		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			

			if ( RxMsg.id == 0x45D)
			{
				// enter power down
				HW_EnterPowerDown();

				// wait until supply is down
				while(1)
				 {}
			}

			// forward message to CAN2
			CAN_Write ( CAN_BUS2, &RxMsg);
		}
		
		
		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) == CAN_ERR_OK)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			// forward message to CAN1
			CAN_Write ( CAN_BUS1, &RxMsg);
		}
	}
}
